// Copyright (c) 2018-2019, AT&T Intellectual Property. All rights reserved.
// SPDX-License-Identifier: GPL-2.0-only
package forwarding

import (
	"bytes"
	"strings"
	"testing"
)

func TestReadSystemNameserverWithMatch(t *testing.T) {
	const file = `
# file generated by /opt/vyatta/sbin/vyatta_update_resolv.pl do not edit
nameserver      8.8.8.8 # system added by vyatta-system-nameservers
nameserver	10.156.55.193
search example.com
`
	r := strings.NewReader(file)
	ns := readSystemNameservers(r)
	if len(ns) != 1 {
		t.Fatal("didn't get expected number of name servers")

	}
	if ns[0] != "8.8.8.8" {
		t.Fatal("didn't get expected name server")
	}
}

func TestReadSystemNameserverWithMultipleMatches(t *testing.T) {
	const file = `
# file generated by /opt/vyatta/sbin/vyatta_update_resolv.pl do not edit
nameserver      8.8.8.8 # system added by vyatta-system-nameservers
nameserver      8.8.4.4 # system added by vyatta-system-nameservers
nameserver	10.156.55.193
search example.com
`
	r := strings.NewReader(file)
	ns := readSystemNameservers(r)
	if len(ns) != 2 {
		t.Fatal("didn't get expected number of name servers")

	}
	if ns[0] != "8.8.8.8" {
		t.Fatal("didn't get expected name server")
	}
	if ns[1] != "8.8.4.4" {
		t.Fatal("didn't get expected name server")
	}
}

func TestReadSystemNameserverNoMatches(t *testing.T) {
	const file = `
# file generated by /opt/vyatta/sbin/vyatta_update_resolv.pl do not edit
nameserver      8.8.8.8
nameserver	10.156.55.193
search example.com
`
	r := strings.NewReader(file)
	ns := readSystemNameservers(r)
	if len(ns) != 0 {
		t.Fatal("didn't get expected number of name servers")

	}
}

func TestWriteDnsmasqSystemConfig(t *testing.T) {
	const expected = `### Autogenerated by vci-service-dns
### Note: Manual changes to this file will be lost during
###       the next commit.
server=8.8.8.8	# system
server=8.8.4.4	# system
`
	var buf bytes.Buffer
	err := writeDnsmasqSystemConfig(&buf, []string{"8.8.8.8", "8.8.4.4"})
	if err != nil {
		t.Fatal(err)
	}
	if buf.String() != expected {
		t.Fatal("didn't get expected output")
	}
}
